#include <string.h>
#include <stdlib.h>
#include "loadfile.h"

/* SORT_TABLE -- Sort a table on a specified column
 *
 * i: tab	table descriptor
 * i: colname	column name
 */

table *sort_table (table *tab, char *colname) {

    table	*sort;
    int		col, ikey, icol, irow, jrow, krow;
    char	*kname, *value, *minval, **cname;

    /* Check column name to see if it exists */

    col = get_tab_colnum (tab, colname);
    if (col == BAD) {
	lod_error = "Table column not found";
	return NULL;
    }

    /* Check for undefined values in sort column */

    for (irow = 0; irow < nrow_table (tab); irow ++) {
	value = get_tab_cell_bynum (tab, col, irow);
	if (value[0] == '\0') {
	    lod_error = "Undefined value in sort column";
	    return NULL;
	}
    }

    /* Create new table, copy keywords and column names */

    sort = create_table ();

    for (ikey = 0; ikey < nkey_table (tab); ikey ++) {
	kname = get_tab_keyname_bynum (tab, ikey);
	value = get_tab_keyval_bynum (tab, ikey);
	put_tab_key (sort, kname, value);
    }

    cname = (char **) malloc (ncol_table (tab) * sizeof (char *));

    for (icol = 0; icol < ncol_table (tab); icol ++) {
	cname[icol] = get_tab_colname (tab, icol);
	add_tab_col (sort, cname[icol]);
    }

    /* Selection sort. Get smallest remaing value 
     * and copy its row to the new table
     */

    for (irow = 0; irow < nrow_table (tab); irow ++) {
	/* get smallest value */

	krow  = 0;
	minval = get_tab_cell_bynum (tab, col, krow);

	for (jrow = 1; jrow < nrow_table (tab); jrow ++) {
	    value = get_tab_cell_bynum (tab, col, jrow);
	    if (strlt (value, minval)) {
		minval = value;
		krow = jrow;
	    }
	}

	/* Copy row */

	for (icol = 0; icol < ncol_table (tab); icol ++) {
	    value = get_tab_cell_bynum (tab, icol, krow);
	    put_tab_cell (sort, cname[icol], irow, value);
	}

	/* Flag value as already copied. Assumes this 
	 * value does not occur in original table
	 */

	minval[0] = '\177';
    }

    free_table (tab); 
    free (cname); cname=NULL;

    return sort;
}
